def merge_paths(first: str, second: str) -> str:
    common_substring = ""
    for i in range(len(first)):
        for j in range(len(second)):
            if first[i:] == second[:j]:
                common_substring = first[i:]
                break

    return first + second[len(common_substring):]



from functools import wraps

from requests import Response

from wapi.features.request import make_request
from wapi.static.T import T
from wapi.static.methods import HTTPMethod
from wapi.utils.get_path import get_path


def PATCH(
        path: str = "",
        _T: T = None
) -> T | Response:
    def decorator(func):
        @wraps(func)
        def wrapper(self, *args, **kwargs):
            p = get_path(self, func, path)

            return make_request(
                url=p,
                method=HTTPMethod.PATCH,
                _T=_T,
                data=kwargs
            )

        return wrapper

    return decorator



def get_used_vars(path: str, params: dict) -> list[str]:
    keys_in_string = [key.strip('{}') for part in path.split('{') for key in part.split('}') if key]
    used_keys = [key for key in keys_in_string if key in params]

    return used_keys




from dataclasses import asdict, is_dataclass

from jsons import load
from requests import request

from wapi.static.T import T
from wapi.static.params import RequestParams
from wapi.utils.get_used_vars import get_used_vars


def make_request(
        url: str,
        method: str,
        _T: T = None,
        data: dict = None,
):
    headers: dict = data.pop(RequestParams.HEADERS, None)
    params: dict = data.pop(RequestParams.PARAMS, None)
    files: dict = data.pop(RequestParams.FILES, None)
    cookies: dict = data.pop(RequestParams.COOKIES, None)
    redirect: bool = data.pop(RequestParams.REDIRECT, True)
    if is_dataclass(
            body := data.pop(RequestParams.BODY, None)
    ):
        body = asdict(body)

    if params is not None:
        used_params = get_used_vars(url, params)
        url = url.format(**params)

        for used_param in used_params:
            params.pop(used_param)

    response = request(
        method=method,
        url=url,
        headers=headers,
        params=params,
        json=body,
        files=files,
        cookies=cookies,
        allow_redirects=redirect
    )

    if _T is None:
        return response
    return load(response.json(), _T)



from typing import Callable


def get_path(cls: type, f: Callable, path: str):
    abs_path = ""

    if hasattr(type(cls), 'path'):
        abs_path += getattr(type(cls), 'path') + abs_path

    if hasattr(f, 'path'):
        abs_path += getattr(f, 'path')

    return abs_path + path



#!/usr/bin/env python

from io import open

from setuptools import setup, find_packages

"""
:authors: WHAOX
:license: MIT License, see LICENSE file
:copyright: (c) 2024 WHAOX
"""

version = '1.1.0'

with open('README.md', encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='whaox-wapi',
    version=version,

    author='WHAOX',
    author_email='gorogannisan641@gmail.com',

    description="Web-Library for Python",
    long_description=long_description,
    long_description_content_type='text/markdown',

    url='https://github.com/topanim/WApi',

    license='MIT LICENSE, see LICENSE file',

    packages=find_packages(),
    install_requires=['jsons', 'requests'],

    keywords=[
        'python',
        'web',
        'api',
        'requests',
        'post',
        'get',
        'put',
        'patch',
        'delete',
        'rest'
    ]
)




class RequestParams:
    HEADERS = "headers"
    PARAMS = "params"
    BODY = "body"
    FILES = "files"
    TIMEOUT = "timeout"
    REDIRECT = "redirect"
    COOKIES = "cookies"

    @classmethod
    def get_params(cls):
        return [
            cls.HEADERS,
            cls.PARAMS,
            cls.BODY,
            cls.FILES,
            cls.TIMEOUT,
            cls.REDIRECT,
            cls.COOKIES
        ]



from wapi.utils.merge_paths import merge_paths


def set_path(obj):
    for name, cls in vars(obj).items():
        if hasattr(cls, 'path'):
            type(cls).path = merge_paths(obj.path, cls.path)
            set_path(type(cls))



from functools import wraps
from typing import Type

from wapi.utils.is_class import is_class
from wapi.utils.set_path import set_path


def Route(path: str):
    def decorator(obj: Type[object]):
        def class_wrapper():
            if hasattr(obj, 'path'):
                obj.path += path
            else:
                obj.path = path

            set_path(obj)

            return obj

        @wraps(obj)
        def func_wrapper(self, *args, **kwargs):
            if hasattr(obj.__wrapped__, 'path'):
                obj.__wrapped__.path = path + obj.__wrapped__.path
            else:
                obj.__wrapped__.path = path

            return obj(self, *args, **kwargs)

        if is_class(obj):
            return class_wrapper()
        return func_wrapper

    return decorator
